@@ -1,10 +1,21 @@
 
 /*
     xskat - a card game for 1 to 3 players.
-    Copyright (C) 1999  Gunter Gerhardt
+    Copyright (C) 2000  Gunter Gerhardt
 
     This program is free software; you can redistribute it freely.
     Use it at your own risk; there is NO WARRANTY.
+
+    Redistribution of modified versions is permitted
+    provided that the following conditions are met:
+    1. All copyright & permission notices are preserved.
+    2.a) Only changes required for packaging or porting are made.
+      or
+    2.b) It is clearly stated who last changed the program.
+         The program is renamed or
+         the version number is of the form x.y.z,
+         where x.y is the version of the original program
+         and z is an arbitrary suffix.
 */
 
 #define IRC_C
@@ -32,7 +43,7 @@ int in;
 
   if (!irc_logfile) return;
   if (!ini) {
-    f=strcmp(irc_logfile,"-")?fopen(irc_logfile,"w"):stdout;
+    f=strcmp(irc_logfile,"-")?fopen(irc_logfile,irc_logappend?"a":"w"):stdout;
     if (!f) {
       fprintf(stderr,"Can't write file %s\n",irc_logfile);
       irc_logfile=0;
@@ -68,6 +79,7 @@ char *s;
       while (fputc(c,stderr)==EOF);
     }
   }
+  fflush(stderr);
 }
 
 VOID irc_printnl(s)
@@ -75,7 +87,6 @@ char *s;
 {
   irc_print(s);
   irc_print("\n");
-  fflush(stdout);
 }
 
 char *irc_getline()
@@ -120,41 +131,42 @@ int irc_xinput(s,l)
 char *s;
 int l;
 {
-  static char buf[1024];
-  static int len;
-
   s[l]=0;
   switch (s[0]) {
   case '\r':
-    if (!len) return 0;
+    if (!irc_inplen) return 0;
     s[0]='\n';
     break;
   case '\b':case 127:
-    strcpy(s,"\b \b");
+    if (irc_inplen) {
+      strcpy(s,"\b \b");
+    }
+    else {
+      s[0]=l=0;
+    }
     break;
   case ' ':
   case '\t':
-    if (!len) return 0;
+    if (!irc_inplen) return 0;
     break;
   case 0x1B:
     return 0;
   }
   irc_print(s);
-  fflush(stdout);
   if (s[0]=='\n') {
-    buf[len]=0;
-    irc_talk(buf);
-    len=0;
+    irc_inpbuf[irc_inplen]=0;
+    irc_talk(irc_inpbuf);
+    irc_inplen=0;
   }
   else if (s[0]=='\b') {
-    if (len) len--;
+    irc_inplen--;
   }
-  else if (len+l<sizeof(buf)){
-    strcpy(buf+len,s);
-    len+=l;
+  else if (irc_inplen+l<sizeof(irc_inpbuf)){
+    strcpy(irc_inpbuf+irc_inplen,s);
+    irc_inplen+=l;
   }
   else {
-    len=0;
+    irc_inplen=0;
   }
   return 1;
 }
@@ -453,7 +465,7 @@ char *msg;
   static int ini,len;
   static char buf[1024],channel[1024],nick[IRC_NICK_LEN+1];
   int i,c;
-  char *p;
+  char *p,plb[80];
 
   irc_checksync();
   irc_checkhist();
@@ -654,14 +666,15 @@ char *msg;
 	      irc_printnl(textarr[TX_NICHT_HIER].t[lang[0]]);
 	    }
 	    else {
-	      irc_server=1;
 	      irc_clients=0;
 	      irc_2player=*p=='2';
 	      irc_out("notice ");
 	      irc_out(irc_channel);
 	      irc_out(" :/server ");
-	      irc_out(textarr[TX_IRC_VERSION].t[lang[0]]);
+	      irc_out(textarr[TX_XSKAT].t[lang[0]]);
 	      irc_out("\n");
+	      sprintf(plb,textarr[TX_WARTEN_AUF_SPIELER_N].t[lang[0]],2);
+	      irc_printnl(plb);
 	      irc_state=IRC_SERVER;
 	    }
 	  }
@@ -682,11 +695,17 @@ char *msg;
 	  irc_printnl(textarr[TX_UNBEKANNTES_KOMMANDO].t[lang[0]]);
 	}
 	else {
-	  irc_out("privmsg ");
-	  irc_out(irc_channel);
-	  irc_out(" :");
-	  irc_out(p);
-	  irc_out("\n");
+	  if (!strcmp(p,"go") || !strcmp(p,"go2")) {
+	    sprintf(plb,textarr[TX_SOLLTE_DAS_GO_X_SEIN].t[lang[0]],p+2);
+	    irc_printnl(plb);
+	  }
+	  else {
+	    irc_out("privmsg ");
+	    irc_out(irc_channel);
+	    irc_out(" :");
+	    irc_out(p);
+	    irc_out("\n");
+	  }
 	}
 	len=0;
       }
@@ -897,7 +916,8 @@ char *q;
 VOID irc_getserverconf(q)
 char *q;
 {
-  static int i;
+  static int i,f;
+  char plb[80];
   int ln;
 
   if (irc_match("1 ",&q)) {
@@ -914,6 +934,7 @@ char *q;
     calc_desk(i);
     if (!strcmp(irc_conf[i].nick,irc_nick)) {
       irc_pos=i;
+      f=1;
     }
     else if (nimmstich[i][0]<101) {
       nimmstich[i][0]=0;
@@ -921,20 +942,27 @@ char *q;
     numsp=++i;
     irc_2player=numsp==2;
   }
-  else if (irc_match("2 ",&q)) {
-    sscanf(q,"%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%ld,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",
+  else if (f && irc_match("2 ",&q)) {
+    sscanf(q,"%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%ld,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",
 	   &playramsch,&playsramsch,&playkontra,&playbock,&resumebock,
 	   &spitzezaehlt,&revolution,&klopfen,&schenken,
 	   &bockevents,&geber,&alist[0],&strateg[0],&oldrules,
 	   &savseed,&bockspiele,&bockinc,&ramschspiele,
 	   &sum[0][0],&sum[0][1],&sum[0][2],
 	   &sum[1][0],&sum[1][1],&sum[1][2],
-	   &sum[2][0],&sum[2][1],&sum[2][2]);
+	   &sum[2][0],&sum[2][1],&sum[2][2],
+	   &cgewoverl[0][0],&cgewoverl[1][0],&cgewoverl[2][0],
+	   &cgewoverl[0][1],&cgewoverl[1][1],&cgewoverl[2][1]);
     savseed-=11;
     setrnd(&seed[0],savseed);
     setsum(0);
+    if (!irc_2player) {
+      sprintf(plb,textarr[TX_VERBUNDEN_MIT_SPIELER_N].t[lang[0]],3);
+      irc_printnl(plb);
+    }
     irc_state=IRC_PLAYING;
   }
+  else i=0;
 }
 
 VOID irc_putserverconf()
@@ -950,7 +978,7 @@ VOID irc_putserverconf()
 	    alist[0],irc_conf[i].nimmstich,irc_conf[i].abkuerz);
     irc_out(buf);
   }
-  sprintf(buf,"notice %s :/svconf2 %d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%ld,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",
+  sprintf(buf,"notice %s :/svconf2 %d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%ld,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",
 	  irc_channel,
 	  playramsch,playsramsch,playkontra,playbock,resumebock,
 	  spitzezaehlt,revolution,klopfen,schenken,
@@ -958,7 +986,9 @@ VOID irc_putserverconf()
 	  savseed+11,bockspiele,bockinc,ramschspiele,
 	  sum[0][0],sum[0][1],sum[0][2],
 	  sum[1][0],sum[1][1],sum[1][2],
-	  sum[2][0],sum[2][1],sum[2][2]);
+	  sum[2][0],sum[2][1],sum[2][2],
+	  cgewoverl[0][0],cgewoverl[1][0],cgewoverl[2][0],
+	  cgewoverl[0][1],cgewoverl[1][1],cgewoverl[2][1]);
   irc_out(buf);
   irc_state=IRC_PLAYING;
 }
@@ -1011,6 +1041,7 @@ VOID irc_getclientconf(s,q)
 char *s,*q;
 {
   int i,ln;
+  char plb[80];
 
   i=irc_clients;
   strcpy(irc_conf[i].nick,s);
@@ -1021,6 +1052,12 @@ char *s,*q;
 	 &irc_conf[i].large,&irc_conf[i].nimmstich,
 	 &irc_conf[i].pos,&irc_conf[i].abkuerz);
   irc_clients=++i;
+  sprintf(plb,textarr[TX_VERBUNDEN_MIT_SPIELER_N].t[lang[0]],i+1);
+  irc_printnl(plb);
+  if (!irc_2player && i<2) {
+    sprintf(plb,textarr[TX_WARTEN_AUF_SPIELER_N].t[lang[0]],i+2);
+    irc_printnl(plb);
+  }
   if (i==1+!irc_2player) {
     numsp=i+1;
     irc_conf[i].pos=irc_pos;
@@ -1105,10 +1142,15 @@ char *oldnick,*newnick;
 VOID irc_msg(nick,q)
 char *nick,*q;
 {
-  int sn;
+  int i,sn;
   char nam[20];
 
   irc_alarm();
+  if (irc_inplen) {
+    for (i=0;i<irc_inplen;i++) {
+      irc_print("\b \b");
+    }
+  }
   irc_print("<");
   irc_print(nick);
   if (irc_state==IRC_PLAYING && (sn=irc_senderok(nick))) {
@@ -1120,12 +1162,17 @@ char *nick,*q;
   }
   irc_print("> ");
   irc_printnl(q);
+  if (irc_inplen) {
+    irc_inpbuf[irc_inplen]=0;
+    irc_print(irc_inpbuf);
+  }
 }
 
 VOID irc_parse(s)
 char *s;
 {
   char *p,*q,nick[IRC_NICK_LEN+1];
+  char plb[80];
   int i,sn,nickerr,colon;
   static int ini;
   static char mynick[IRC_NICK_LEN+1];
@@ -1146,23 +1193,38 @@ char *s;
 	  (q=strchr(q+1,':'))) q++;
       if (q && irc_match("NOTICE",&p)) {
 	if (irc_match("/server ",&q)) {
-	  if (irc_state==IRC_TALK) {
+	  if (irc_state==IRC_SERVER) {
+	    irc_printnl(textarr[TX_NUR_EINER_SOLLTE_GO_SAGEN].t[lang[0]]);
+	    irc_out("notice ");
+	    irc_out(irc_channel);
+	    irc_out(" :/noserver\n");
+	    irc_state=IRC_TALK;
+	  }
+	  else if (irc_state==IRC_TALK) {
 	    irc_out("notice ");
 	    irc_out(irc_channel);
 	    irc_out(" :/client ");
-	    irc_out(textarr[TX_IRC_VERSION].t[lang[0]]);
+	    irc_out(textarr[TX_XSKAT].t[lang[0]]);
 	    irc_out("\n");
-	    if (!irc_match(textarr[TX_IRC_VERSION].t[lang[0]],&q)) {
+	    if (!irc_match(textarr[TX_XSKAT].t[lang[0]],&q)) {
 	      irc_pr_ss(textarr[TX_VERSCHIEDENE_VERSIONEN].t[lang[0]],q);
 	    }
 	    else {
+	      sprintf(plb,textarr[TX_VERBUNDEN_MIT_SPIELER_N].t[lang[0]],2);
+	      irc_printnl(plb);
 	      irc_putclientconf();
 	    }
 	  }
 	}
+	else if (irc_match("/noserver",&q)) {
+	  if (irc_state==IRC_SERVER) {
+	    irc_printnl(textarr[TX_NUR_EINER_SOLLTE_GO_SAGEN].t[lang[0]]);
+	    irc_state=IRC_TALK;
+	  }
+	}
 	else if (irc_match("/client ",&q)) {
 	  if (irc_state==IRC_SERVER) {
-	    if (!irc_match(textarr[TX_IRC_VERSION].t[lang[0]],&q)) {
+	    if (!irc_match(textarr[TX_XSKAT].t[lang[0]],&q)) {
 	      irc_pr_ss(textarr[TX_VERSCHIEDENE_VERSIONEN].t[lang[0]],q);
 	      irc_state=IRC_TALK;
 	    }
@@ -1340,8 +1402,9 @@ VOID irc_init()
   xinitplayers();
   irc_connect();
   irc_checknick();
-  irc_print("XSkat IRC ");
-  irc_printnl(textarr[TX_IRC_VERSION].t[lang[0]]);
+  irc_print("*** ");
+  irc_print(textarr[TX_XSKAT].t[lang[0]]);
+  irc_printnl(" ***");
   irc_printnl(textarr[TX_README_IRC].t[lang[0]]);
   irc_print("Login ");
   irc_print(irc_user);
