@@ -1,10 +1,21 @@
 
 /*
     xskat - a card game for 1 to 3 players.
-    Copyright (C) 1999  Gunter Gerhardt
+    Copyright (C) 2000  Gunter Gerhardt
 
     This program is free software; you can redistribute it freely.
     Use it at your own risk; there is NO WARRANTY.
+
+    Redistribution of modified versions is permitted
+    provided that the following conditions are met:
+    1. All copyright & permission notices are preserved.
+    2.a) Only changes required for packaging or porting are made.
+      or
+    2.b) It is clearly stated who last changed the program.
+         The program is renamed or
+         the version number is of the form x.y.z,
+         where x.y is the version of the original program
+         and z is an arbitrary suffix.
 */
 
 #ifndef SKAT_H
@@ -109,10 +120,12 @@ EXTERN int lang[3],langset[3];
 EXTERN int maxrw[3],sort1[3],sort2[3],alternate[3],splfirst[3];
 EXTERN int downupset[3],altseqset[3],tdelayset[3],alistset[3];
 EXTERN int mbutton[3],mbuttonset[3];
+EXTERN int keyboard[3],keyboardset[3];
 EXTERN int abkuerz[3],abkuerzset[3];
 EXTERN int useoptfile[3];
 EXTERN int lastmsaho[3],protsort[3],hatnfb[3][5];
 EXTERN int splsum[3][3],sum[3][3],prevsum[3][3],alist[3];
+EXTERN int sgewoverl[3][2],cgewoverl[3][2];
 EXTERN int nimmstich[3][2],playramsch,ramschset;
 EXTERN int playsramsch,sramschset,sramschstufe;
 EXTERN int playkontra,kontraset,kontrastufe;
