@@ -1,23 +1,34 @@
 
 /*
     xskat - a card game for 1 to 3 players.
-    Copyright (C) 1999  Gunter Gerhardt
+    Copyright (C) 2000  Gunter Gerhardt
 
     This program is free software; you can redistribute it freely.
     Use it at your own risk; there is NO WARRANTY.
+
+    Redistribution of modified versions is permitted
+    provided that the following conditions are met:
+    1. All copyright & permission notices are preserved.
+    2.a) Only changes required for packaging or porting are made.
+      or
+    2.b) It is clearly stated who last changed the program.
+         The program is renamed or
+         the version number is of the form x.y.z,
+         where x.y is the version of the original program
+         and z is an arbitrary suffix.
 */
 
 #define TEXT_C
 
+#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
 #include "defs.h"
 #include "text.h"
 
-#define VERSION   " X S K A T   3.2 "
-#define IRC_VERS  "1.2"
-#define COPYRIGHT "Copyright 1999  Gunter Gerhardt"
+#define VERSION   " X S K A T   3.3 "
+#define COPYRIGHT "Copyright 2000  Gunter Gerhardt"
 #define EMAIL     "(gerhardt@draeger.com)"
 #define HOMEPAGE  "XSkat Home Page"
 #define URL       "http://www.gulu.net/xskat/"
@@ -51,6 +62,7 @@ static char *ger_text[]={
   "Null Hand:35  Null ouvert Hand:59",
   "Revolution:92",
   " Angesagt ist "," Resultat ","Der Spielwert ist","Ende",
+  "Spiele gewonnen/verloren",
   "Protokoll","Weiter"," Protokoll ",
   "Im Skat ist:",
   "Im Skat war:",
@@ -87,7 +99,8 @@ static char *ger_text[]={
   "Erfolgreicher Kontra","Kontra & Re angesagt",
   "NNN Punkte in Spielliste","N00 Punkte in Spielliste",
   "Spielwert ist >= +72","Spielwert ist >= +96",
-  " Men-Button ","Jeder","und ESC / F1",
+  " Eingabe ","Tastatur",
+  "Men-Button","Jeder","und ESC / F1",
   "Spitze","zhlt 2","Spitze verloren !","Spitze !",
   "Spitze nicht erlaubt !",
   "Niedrigster Trumpf nicht auf der Hand.",
@@ -108,9 +121,13 @@ static char *ger_text[]={
   "IRC-Verbindung herstellen mit:",
   "Zur IRC-Konfiguration siehe:",
   "man xskat",
-  IRC_VERS,"Verschiedene Versionen",
+  "Verschiedene Versionen",
   "(Probleme? README.IRC oder 'man xskat' NOTES lesen!)",
   "Nicht auf diesem Kanal !",
+  "Nur ein Spieler sollte /go sagen !  Nochmal.",
+  "Sollte das /go%s sein ?",
+  "Verbunden mit Spieler%d.",
+  "Warten auf Spieler%d.",
   "\n Zum Spielen mit /join #xskatXYZ auf einen freien Kanal wechseln.",
   " Wenn alle Mitspieler auf diesem Kanal sind, mu einer /go eingeben,",
   " bzw. /go2, wenn nur 2 Spieler teilnehmen.",
@@ -166,6 +183,7 @@ static char *eng_text[]={
   "Null Hand:35  Null ouvert Hand:59",
   "Revolution:92",
   " Playing "," Result ","The value of the game is","Quit",
+  "Games won/lost",
   "Log","Continue"," Log ",
   "Skat is:",
   "Skat was:",
@@ -202,7 +220,8 @@ static char *eng_text[]={
   "Successful Kontra","Kontra & Re game",
   "NNN points in game list","N00 points in game list",
   "Game value is >= +72","Game value is >= +96",
-  " Menu button ","Any","and ESC / F1",
+  " Input ","Keyboard",
+  "Menu button","Any","and ESC / F1",
   "Spitze","counts 2","Lost Spitze !","Spitze !",
   "Spitze not allowed !",
   "Lowest trump not in your hand.",
@@ -223,9 +242,13 @@ static char *eng_text[]={
   "Establish IRC connection with:",
   "For IRC configuration see:",
   "man xskat",
-  IRC_VERS,"Version mismatch",
+  "Version mismatch",
   "(Problems? See README.IRC or 'man xskat' NOTES!)",
   "Not on this channel !",
+  "Only one player should say /go !  Try again.",
+  "Did you mean /go%s ?",
+  "Connected to Player%d.",
+  "Waiting for Player%d.",
   "\n Start playing by typing /join #xskatXYZ to switch to a free channel.",
   " When all players are on this channel, one player has to enter /go,",
   " or /go2, if there are only 2 players.",
@@ -257,9 +280,10 @@ tx_typ textarr[TX_NUM_TX];
 static struct {
   char **arr;
   char *name;
+  char *langpref[5];
 } textdesc[NUM_LANG] = {
-  {ger_text,"german"},
-  {eng_text,"english"}
+  {ger_text,"german",{"de","german",0}},
+  {eng_text,"english",{"en",0}}
 };
 
 VOID init_text()
@@ -277,7 +301,7 @@ int langidx(s)
 char *s;
 {
   char h[80];
-  int i;
+  int i,j;
 
   for (i=0;i<79 && s && *s;i++,s++) {
     h[i]=tolower(*s);
@@ -294,6 +318,19 @@ char *s;
     }
     fputs(".\n",stderr);
   }
+  s=getenv("LANG");
+  if (s) {
+    for (i=0;i<79 && s && *s;i++,s++) {
+      h[i]=tolower(*s);
+    }
+    h[i]=0;
+    for (i=0;i<NUM_LANG;i++) {
+      for (j=0;j<5 && textdesc[i].langpref[j];j++) {
+	if (!strncmp(h,textdesc[i].langpref[j],
+		     strlen(textdesc[i].langpref[j]))) return i;
+      }
+    }
+  }
   for (i=0;i<NUM_LANG;i++) {
     if (!strcmp(textdesc[i].name,DEFAULT_LANGUAGE)) return i;
   }
