@@ -1,10 +1,21 @@
 
 /*
     xskat - a card game for 1 to 3 players.
-    Copyright (C) 1999  Gunter Gerhardt
+    Copyright (C) 2000  Gunter Gerhardt
 
     This program is free software; you can redistribute it freely.
     Use it at your own risk; there is NO WARRANTY.
+
+    Redistribution of modified versions is permitted
+    provided that the following conditions are met:
+    1. All copyright & permission notices are preserved.
+    2.a) Only changes required for packaging or porting are made.
+      or
+    2.b) It is clearly stated who last changed the program.
+         The program is renamed or
+         the version number is of the form x.y.z,
+         where x.y is the version of the original program
+         and z is an arbitrary suffix.
 */
 
 #define XDIAL_C
@@ -25,6 +36,24 @@
 #include "text.h"
 #include "xdial.h"
 
+#ifndef XK_KP_Tab
+#define XK_KP_Tab           0xFF89
+#endif
+#ifndef XK_KP_Enter
+#define XK_KP_Enter         0xFF8D
+#endif
+#ifndef XK_KP_Left
+#define XK_KP_Left          0xFF96
+#endif
+#ifndef XK_KP_Up
+#define XK_KP_Up            0xFF97
+#endif
+#ifndef XK_KP_Right
+#define XK_KP_Right         0xFF98
+#endif
+#ifndef XK_KP_Down
+#define XK_KP_Down          0xFF99
+#endif
 #ifndef XK_3270_BackTab
 #define XK_3270_BackTab     0xFD05
 #endif
@@ -70,7 +99,7 @@ VOID info_reiz()
 
   for (ln=0;ln<NUM_LANG;ln++) tt.t[ln]=txt[ln];
   for (sn=0;sn<numsp;sn++) {
-    if (reizp<0) {
+    if (reizp<0 || ramschspiele) {
       dioptions[sn][11].str=OB_NONE;
     }
     else {
@@ -193,7 +222,7 @@ VOID init_dials()
   INIT_DI(divarianten);
   INIT_DI(dibockevents);
   INIT_DI(digeschwindigkeit);
-  INIT_DI(dimenubutton);
+  INIT_DI(dieingabe);
   INIT_DI(diweiter);
   INIT_DI(dikontra);
   INIT_DI(dikonre);
@@ -522,9 +551,9 @@ int sn,bt;
     else if (ob==diresult) {
       remove_di(sn);
       clr_desk(0);
-      if (bt==16) di_ende(sn);
-      else if (bt==18) di_proto(sn,1,0);
-      else if (bt==17) di_wieder(sn);
+      if (bt==20) di_ende(sn);
+      else if (bt==22) di_proto(sn,1,0);
+      else if (bt==21) di_wieder(sn);
       else phase=GEBEN;
     }
     else if (ob==diwieder) {
@@ -551,18 +580,18 @@ int sn,bt;
     }
     else if (ob==diliste[sn]) {
       remove_di(sn);
-      if (bt==59) {
+      if (bt==62) {
 	splfirst[sn]-=12;
 	di_liste(sn,0);
-	actbtn[sn]=splfirst[sn]?59:62;
+	actbtn[sn]=splfirst[sn]?62:65;
       }
-      else if (bt==60) {
+      else if (bt==63) {
 	alist[sn]=(alist[sn]+1)%3;
 	if (!sn) save_opt();
 	di_liste(sn,splfirst[sn]+12>=splstp);
-	actbtn[sn]=60;
+	actbtn[sn]=63;
       }
-      else if (bt==62) {
+      else if (bt==65) {
 	if (splfirst[sn]+12>=splstp) {
 	  di_loesch(sn);
 	}
@@ -574,7 +603,7 @@ int sn,bt;
 	  else {
 	    di_liste(sn,1);
 	  }
-	  actbtn[sn]=splfirst[sn]+12<splstp?62:59;
+	  actbtn[sn]=splfirst[sn]+12<splstp?65:62;
 	}
       }
       else if (phase==RESULT) di_resultdi(sn);
@@ -782,7 +811,7 @@ int sn,bt;
       if (bt==17) {
 	remove_di(sn);
 	if (!irc_play && numsp==1) di_irc(sn);
-	else di_menubutton(sn);
+	else di_eingabe(sn);
       }
       else {
 	draw_di(sn,3);
@@ -790,21 +819,26 @@ int sn,bt;
       }
     }
     else if (ob==diirc) {
-      if (bt==6) di_menubutton(sn);
+      if (bt==6) di_eingabe(sn);
       else {
 	if (!fork()) startirc(1);
 	exitus(0);
       }
     }
-    else if (ob==dimenubutton[sn]) {
+    else if (ob==dieingabe[sn]) {
       remove_di(sn);
-      sav=mbutton[sn];
+      sav=keyboard[sn]+3*mbutton[sn];
       for (i=0;i<=5;i++) {
-	if (dimenubutton[sn][2+i].spec&OB_SELECTED) {
+	if (dieingabe[sn][3+i].spec&OB_SELECTED) {
 	  mbutton[sn]=i;
 	}
       }
-      if (sav!=mbutton[sn] && !irc_play) save_opt();
+      for (i=0;i<3;i++) {
+	if (dieingabe[sn][11+i].spec&OB_SELECTED) {
+	  keyboard[sn]=i;
+	}
+      }
+      if (sav!=keyboard[sn]+3*mbutton[sn] && !irc_play) save_opt();
     }
   }
 }
@@ -826,7 +860,7 @@ OBJECT *ob;
       ob!=divarianten[sn] &&
       ob!=dibockevents[sn] &&
       ob!=digeschwindigkeit[sn] &&
-      ob!=dimenubutton[sn]) irc_sendbtev(sn,bt);
+      ob!=dieingabe[sn]) irc_sendbtev(sn,bt);
   hndl_btevent(sn,bt);
 }
 
@@ -868,6 +902,7 @@ int sn,f;
   unsigned long bp;
   int bt,d,e,w,h;
 
+  if (f && !keyboard[sn]) actbtn[sn]=0;
   ob=actdial[sn];
   bt=actbtn[sn];
   if (bt && ob) {
@@ -969,7 +1004,7 @@ int sn;
   static int tc[3],ph[3];
   static long gn[3];
 
-  if (selpos[sn].num || actdial[sn]) return;
+  if (selpos[sn].num || actdial[sn] || !keyboard[sn]) return;
   if (selpos[sn].act==-1 || !(selpos[sn].p[selpos[sn].act].f&4) ||
       gn[sn]!=gamenr || ph[sn]!=phase) {
     selpos[sn].act=0;
@@ -1010,7 +1045,7 @@ int sn;
   else if (phase==SPIELEN) {
     if (sn==(ausspl+vmh)%3) {
       computer();
-      if (actdial[sn]) return;
+      if (actdial[sn] || keyboard[sn]!=2) return;
       calc_poss(sn);
       selpos[sn].num=possc;
       i=0;
@@ -2058,7 +2093,7 @@ VOID di_spiel()
 VOID list_fun(sn)
 int sn;
 {
-  int i,j,k,s,e,r,d,curr[3][3],cp,ln;
+  int i,j,k,s,e,r,d,curr[3][3],cgv[3][2],cp,ln;
   static char txt[3][13][4][10];
   static tx_typ tt[3][13][4];
 
@@ -2066,6 +2101,9 @@ int sn;
     for (j=0;j<3;j++) {
       curr[i][j]=splsum[i][j];
     }
+    for (j=0;j<2;j++) {
+      cgv[i][j]=sgewoverl[i][j];
+    }
     for (j=0;j<13;j++) {
       for (k=0;k<4;k++) {
 	for (ln=0;ln<NUM_LANG;ln++) {
@@ -2076,14 +2114,14 @@ int sn;
   }
   cp=alist[sn];
   for (j=0;j<splfirst[sn];j++) {
-    modsum(curr,j,(int *)0,(int *)0,(int *)0,(int *)0);
+    modsum(curr,cgv,j,(int *)0,(int *)0,(int *)0,(int *)0);
   }
   for (k=0;k<3;k++) {
     diliste[sn][8+k].str=&tt[sn][0][k];
     sprintf(txt[sn][0][k],"%d",curr[k][cp]);
   }
   for (i=splfirst[sn],j=1;j<13 && i<splstp;i++,j++) {
-    modsum(curr,i,&s,&e,&r,&d);
+    modsum(curr,cgv,i,&s,&e,&r,&d);
     for (k=0;k<4;k++) {
       diliste[sn][7+4*j+k].str=&tt[sn][j][k];
       sprintf(txt[sn][j][k],"%d",k==3?e>0 && r && !d?-e:e:curr[k][cp]);
@@ -2103,6 +2141,9 @@ int sn;
       diliste[sn][7+4*j+k].str=OB_NONE;
     }
   }
+  for (k=0;k<3;k++) {
+    sprintf(diliste[sn][59+k].str->t[0],"%d/%d",cgv[k][0],cgv[k][1]);
+  }
 }
 
 VOID di_liste(sn,ini)
@@ -2110,22 +2151,29 @@ int sn,ini;
 {
   int ln;
   static char txt[3][10];
-  static tx_typ tt1[3],tt2,tt3;
+  static char spt[3][3][40];
+  static tx_typ tt1[3],tt2,tt3,tt4[3];
 
   for (ln=0;ln<NUM_LANG;ln++) {
     tt1[sn].t[ln]=txt[sn];
     tt2.t[ln]="<-";
     tt3.t[ln]="->";
+    tt4[0].t[ln]=spt[sn][0];
+    tt4[1].t[ln]=spt[sn][1];
+    tt4[2].t[ln]=spt[sn][2];
   }
   sprintf(txt[sn],"<%d>",alist[sn]+1);
   set_names(diliste[sn],2);
   if (ini) splfirst[sn]=((splstp>0?splstp-1:0)/12)*12;
-  diliste[sn][59].str=&tt2;
-  diliste[sn][59].spec=splfirst[sn]>=12?OB_EXIT:OB_HIDDEN;
-  diliste[sn][60].str=&tt1[sn];
-  diliste[sn][60].spec=irc_play?OB_DISABLED|OB_EXIT:OB_EXIT;
-  diliste[sn][62].str=splfirst[sn]+12>=splstp?&textarr[TX_LOESCHEN]:&tt3;
-  diliste[sn][62].spec=splfirst[sn]+12<splstp ||
+  diliste[sn][59].str=&tt4[0];
+  diliste[sn][60].str=&tt4[1];
+  diliste[sn][61].str=&tt4[2];
+  diliste[sn][62].str=&tt2;
+  diliste[sn][62].spec=splfirst[sn]>=12?OB_EXIT:OB_HIDDEN;
+  diliste[sn][63].str=&tt1[sn];
+  diliste[sn][63].spec=irc_play?OB_DISABLED|OB_EXIT:OB_EXIT;
+  diliste[sn][65].str=splfirst[sn]+12>=splstp?&textarr[TX_LOESCHEN]:&tt3;
+  diliste[sn][65].spec=splfirst[sn]+12<splstp ||
     ((splstp || (irc_play &&
 		 (sum[0][0] || sum[0][1] || sum[0][2] ||
 		  sum[1][0] || sum[1][1] || sum[1][2] ||
@@ -2136,7 +2184,7 @@ int sn,ini;
      ini)?OB_EXIT:OB_HIDDEN;
   list_fun(sn);
   create_di(sn,diliste[sn]);
-  actbtn[sn]=61;
+  actbtn[sn]=64;
 }
 
 int ger_toupper(c)
@@ -2373,7 +2421,7 @@ int sn,ini,log;
     tmhschob[sn].t[ln]=mhschob[sn][ln];
     tbis[sn].t[ln]=bis[sn];
     taug[sn].t[ln]=aug[sn][ln];
-    sprintf(aug[sn][ln],textarr[TX_AUGEN].t[ln],prot1.augen);
+    sprintf(aug[sn][ln],textarr[TX_N_AUGEN].t[ln],prot1.augen);
     im_skat(sn,ln,imskw[sn][ln],0);
     im_skat(sn,ln,imski[sn][ln],1);
     im_skat(sn,ln,vhschob[sn][ln],2);
@@ -2539,8 +2587,8 @@ int be;
   int ln,sn,i,x,y;
   static int ini,smlh;
   static char sa[NUM_LANG][30],sw[NUM_LANG][40],sg[NUM_LANG][40];
-  static char su[3][3][10],txt[NUM_LANG][30];
-  static tx_typ tsa,tsw,tsg,tsu[3][3],ttxt;
+  static char su[3][3][10],txt[NUM_LANG][30],spt[3][40];
+  static tx_typ tsa,tsw,tsg,tsu[3][3],ttxt,tsp[3];
 
   for (sn=0;sn<numsp;sn++) {
     if (handsp || sn!=spieler || trumpf==5) {
@@ -2571,12 +2619,18 @@ int be;
   diresult[3].str=&tsg;
   diresult[4].str=&tsa;
   diresult[5].str=&tsw;
-  diresult[15].str=be?&ttxt:OB_NONE;
+  diresult[16].str=&tsp[0];
+  diresult[17].str=&tsp[1];
+  diresult[18].str=&tsp[2];
+  diresult[19].str=be?&ttxt:OB_NONE;
   for (ln=0;ln<NUM_LANG;ln++) {
     tsg.t[ln]=sg[ln];
     tsa.t[ln]=sa[ln];
     tsw.t[ln]=sw[ln];
     ttxt.t[ln]=txt[ln];
+    tsp[0].t[ln]=spt[0];
+    tsp[1].t[ln]=spt[1];
+    tsp[2].t[ln]=spt[2];
     if (trumpf==5 && spwert==0) {
       strcpy(sg[ln],textarr[TX_NIEMAND].t[ln]);
     }
@@ -2599,14 +2653,17 @@ int be;
 	strcpy(sa[ln],textarr[TX_DAS_SPIEL_SCHWARZ].t[ln]);
       }
       else {
-	sprintf(sa[ln],textarr[TX_MIT_AUGEN].t[ln],stsum,120-stsum);
+	sprintf(sa[ln],textarr[TX_MIT_N_ZU_M_AUGEN].t[ln],stsum,120-stsum);
       }
     }
     sprintf(sw[ln],"%s %d.",textarr[TX_DER_SPIELWERT_IST].t[ln],
 	    spgew && (trumpf!=5 || stsum==120)?spwert:-spwert);
-    if (be>1) sprintf(txt[ln],textarr[TX_BOCK_EREIGNISSE].t[ln],be);
+    if (be>1) sprintf(txt[ln],textarr[TX_N_BOCK_EREIGNISSE].t[ln],be);
     else strcpy(txt[ln],textarr[TX_BOCK_EREIGNIS].t[ln]);
   }
+  for (i=0;i<3;i++) {
+    sprintf(diresult[16+i].str->t[0],"%d/%d",cgewoverl[i][0],cgewoverl[i][1]);
+  }
   for (sn=0;sn<numsp;sn++) {
     for (i=0;i<3;i++) {
       sprintf(su[sn][i],"%d",sum[i][alist[sn]]);
@@ -2624,17 +2681,20 @@ int be;
       dismlres[sn][11].str=diresult[12].str;
       dismlres[sn][12].str=diresult[13].str;
       dismlres[sn][13].str=diresult[14].str;
-      dismlres[sn][14].str=diresult[15].str;
+      dismlres[sn][14].str=diresult[16].str;
+      dismlres[sn][15].str=diresult[17].str;
+      dismlres[sn][16].str=diresult[18].str;
+      dismlres[sn][17].str=diresult[19].str;
       if (!ini) {
 	smlh=dismlres[sn][0].h;
 	ini=1;
       }
       if (be) {
-	dismlres[sn][14].spec&=~OB_HIDDEN;
+	dismlres[sn][17].spec&=~OB_HIDDEN;
 	dismlres[sn][0].h=smlh+1;
       }
       else {
-	dismlres[sn][14].spec|=OB_HIDDEN;
+	dismlres[sn][17].spec|=OB_HIDDEN;
 	dismlres[sn][0].h=smlh;
       }
       di_delres(sn);
@@ -2852,21 +2912,22 @@ int sn;
   diirc[3].str=&tt;
 }
 
-VOID di_menubutton(sn)
+VOID di_eingabe(sn)
 int sn;
 {
   static char txt[5][2];
   static tx_typ tt[5];
   int i,ln;
 
-  create_diopt(sn,dimenubutton[sn]);
-  dimenubutton[sn][2+mbutton[sn]].spec|=OB_SELECTED;
+  create_diopt(sn,dieingabe[sn]);
+  dieingabe[sn][3+mbutton[sn]].spec|=OB_SELECTED;
+  dieingabe[sn][11+keyboard[sn]].spec|=OB_SELECTED;
   for (i=0;i<5;i++) {
     for (ln=0;ln<NUM_LANG;ln++) {
       tt[i].t[ln]=txt[i];
     }
     sprintf(txt[i],"%d",i+1);
-    dimenubutton[sn][3+i].str=&tt[i];
+    dieingabe[sn][4+i].str=&tt[i];
   }
 }
 
