@@ -1,10 +1,21 @@
 
 /*
     xskat - a card game for 1 to 3 players.
-    Copyright (C) 1999  Gunter Gerhardt
+    Copyright (C) 2000  Gunter Gerhardt
 
     This program is free software; you can redistribute it freely.
     Use it at your own risk; there is NO WARRANTY.
+
+    Redistribution of modified versions is permitted
+    provided that the following conditions are met:
+    1. All copyright & permission notices are preserved.
+    2.a) Only changes required for packaging or porting are made.
+      or
+    2.b) It is clearly stated who last changed the program.
+         The program is renamed or
+         the version number is of the form x.y.z,
+         where x.y is the version of the original program
+         and z is an arbitrary suffix.
 */
 
 #ifndef XDIAL_H
@@ -397,21 +408,25 @@ EXTERN OBJECT diresult[]
 #ifdef XDIAL_C
 =
 {
-  {20,0,0,37,17,OB_NONE,OB_NONE},
+  {24,0,0,37,17,OB_NONE,OB_NONE},
   {OB_CENTERED|OB_UNDERLINED,1,1,35,1,TOB(TX_RESULTAT),OB_NONE},
   {OB_CENTERED|OB_BOLD,1,2,35,1,OB_NONE,OB_NONE},
   {OB_CENTERED,1,3,35,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,1,5,35,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,1,7,35,1,TOB(TX_DER_SPIELWERT_IST),OB_NONE},
-  {OB_CENTERED,4,9,10,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,14,9,10,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,24,9,10,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,4,10,10,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,14,10,10,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,24,10,10,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,5,11,8,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,15,11,8,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,25,11,8,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,1,4,35,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,1,5,35,1,TOB(TX_DER_SPIELWERT_IST),OB_NONE},
+  {OB_CENTERED,3,7,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,14,7,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,25,7,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,3,8,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,14,8,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,25,8,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,3,9,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,14,9,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,25,9,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,1,10,35,1,TOB(TX_SPIELE_GEWONNEN_VERLOREN),OB_NONE},
+  {OB_CENTERED,3,11,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,14,11,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,25,11,9,1,OB_NONE,OB_NONE},
   {OB_CENTERED|OB_BOLD,1,12,35,1,OB_NONE,OB_NONE},
   {OB_EXIT,3,13,14,1,TOB(TX_ENDE),OB_NONE},
   {OB_EXIT,20,13,14,1,TOB(TX_WIEDER),OB_NONE},
@@ -421,25 +436,28 @@ EXTERN OBJECT diresult[]
 #endif
 ;
 
-EXTERN OBJECT dismlres[3][15]
+EXTERN OBJECT dismlres[3][18]
 #ifdef XDIAL_C
 =
 {{
-  {15,0,0,33,8,OB_NONE,OB_NONE},
+  {18,0,0,33,9,OB_NONE,OB_NONE},
   {OB_CENTERED|OB_BOLD,1,0,31,1,OB_NONE,OB_NONE},
   {OB_CENTERED,1,1,31,1,OB_NONE,OB_NONE},
   {OB_CENTERED,1,2,31,1,OB_NONE,OB_NONE},
   {OB_CENTERED,1,3,31,1,TOB(TX_DER_SPIELWERT_IST),OB_NONE},
-  {OB_CENTERED,2,4,10,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,12,4,10,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,22,4,10,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,2,5,10,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,12,5,10,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,22,5,10,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,3,6,8,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,13,6,8,1,OB_NONE,OB_NONE},
-  {OB_CENTERED,23,6,8,1,OB_NONE,OB_NONE},
-  {OB_CENTERED|OB_BOLD,1,7,31,1,OB_NONE,OB_NONE}
+  {OB_CENTERED,2,4,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,12,4,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,22,4,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,2,5,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,12,5,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,22,5,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,2,6,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,12,6,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,22,6,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,2,7,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,12,7,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,22,7,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_BOLD,1,8,31,1,OB_NONE,OB_NONE}
 }}
 #endif
 ;
@@ -502,11 +520,11 @@ EXTERN OBJECT diproto[3][50]
 #endif
 ;
 
-EXTERN OBJECT diliste[3][63]
+EXTERN OBJECT diliste[3][66]
 #ifdef XDIAL_C
 =
 {{
-  {63,0,0,38,21,OB_NONE,OB_NONE},
+  {66,0,0,38,21,OB_NONE,OB_NONE},
   {OB_CENTERED|OB_UNDERLINED,1,1,36,1,TOB(TX_SPIELLISTE_T),OB_NONE},
   {OB_CENTERED,2,3,10,1,OB_NONE,OB_NONE},
   {OB_CENTERED,12,3,10,1,OB_NONE,OB_NONE},
@@ -565,6 +583,9 @@ EXTERN OBJECT diliste[3][63]
   {OB_RIGHT,9,17,10,1,OB_NONE,OB_NONE},
   {OB_RIGHT,19,17,10,1,OB_NONE,OB_NONE},
   {OB_RIGHT,29,17,7,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,3,18,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,13,18,9,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,23,18,9,1,OB_NONE,OB_NONE},
   {OB_EXIT,2,19,8,1,OB_NONE,OB_NONE},
   {OB_EXIT,11,19,5,1,OB_NONE,OB_NONE},
   {OB_EXIT,17,19,10,1,TOB(TX_WEITER),OB_NONE},
@@ -783,20 +804,25 @@ EXTERN OBJECT diirc[]
 #endif
 ;
 
-EXTERN OBJECT dimenubutton[3][10]
+EXTERN OBJECT dieingabe[3][15]
 #ifdef XDIAL_C
 =
 {{
-  {10,0,0,38,9,OB_NONE,OB_NONE},
-  {OB_CENTERED|OB_UNDERLINED,1,1,36,1,TOB(TX_MENU_BUTTON),OB_NONE},
-  {OB_BUTTON,3,3,7,1,TOB(TX_JEDER),1},
-  {OB_BUTTON,11,3,4,1,OB_NONE,1},
-  {OB_BUTTON,16,3,4,1,OB_NONE,1},
-  {OB_BUTTON,21,3,4,1,OB_NONE,1},
-  {OB_BUTTON,26,3,4,1,OB_NONE,1},
-  {OB_BUTTON,31,3,4,1,OB_NONE,-5},
+  {15,0,0,38,12,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,36,1,TOB(TX_EINGABE),OB_NONE},
+  {OB_CENTERED,1,3,36,1,TOB(TX_MENU_BUTTON),OB_NONE},
+  {OB_BUTTON,3,4,7,1,TOB(TX_JEDER),1},
+  {OB_BUTTON,11,4,4,1,OB_NONE,1},
+  {OB_BUTTON,16,4,4,1,OB_NONE,1},
+  {OB_BUTTON,21,4,4,1,OB_NONE,1},
+  {OB_BUTTON,26,4,4,1,OB_NONE,1},
+  {OB_BUTTON,31,4,4,1,OB_NONE,-5},
   {OB_CENTERED,1,5,36,1,TOB(TX_UND_ESC_F1),OB_NONE},
-  {OB_EXIT,12,7,14,1,TOB(TX_WEITER),OB_NONE}
+  {OB_CENTERED,1,7,36,1,TOB(TX_TASTATUR),OB_NONE},
+  {OB_BUTTON,3,8,10,1,TOB(TX_NEIN),1},
+  {OB_BUTTON,14,8,10,1,TOB(TX_JA),1},
+  {OB_BUTTON,25,8,10,1,TOB(TX_IMMER),-2},
+  {OB_EXIT,12,10,14,1,TOB(TX_WEITER),OB_NONE}
 }}
 #endif
 ;
